import numpy as np
import matplotlib.pyplot as plt
import starlink.Ast
import starlink.Grf

#  Create a figure covering the whole work space
fig = plt.figure()

#  Create a plotting region ( matplotlib calls it an "Axes") covering the
#  whole figure. Ensure that the matplotlib axis annotations are not drawn.
ax = fig.add_subplot(111)
ax.xaxis.set_visible(False)
ax.yaxis.set_visible(False)

#  Create a grf module that knows how to draw into the matplotlib plotting
#  region.
grf = starlink.Grf.grf_matplotlib( ax )

#  Find the coords at the centre of the plotting region, and its half-width
#  and half-height.
xlo,xhi = ax.get_xbound()
ylo,yhi = ax.get_ybound()
xcen = ( xlo + xhi )/2
ycen = ( ylo + yhi )/2
w = 0.5*(xhi - xlo)
h = 0.5*(yhi - ylo)

#  The AST Plot will cover half the matplotlib plotting region on ach axis.
gbox = (xcen-w/2,ycen-h/2,xcen+w/2,ycen+h/2)

bbox = (-100,-100,100,100)

#  Create the AST Plot describing coords from -100 to +100 on each axis,
#  linearly related to the matplotlib coords. Use thegrf module created
#  above.
plot = starlink.Ast.Plot( None, gbox, bbox, grf )

#  Draw a coordinate grid, using the matplotlib default graphics
#  properties.
plot.grid()

#  Set markers to use the default colour 1 provided by the grf module,
#  then plot a set of threee markers using marker type 5 (the default
#  type 5 marker is also defined in the grf module).
plot.Colour_Markers = 1
plot.mark( [[-80,0,80],[80,0,-80]], 5)

#  Draw a thick line.
plot.Width_Curves = 10
plot.curve( [-80,-80],[80,80])

#  Plot a string using the default font 0 provided by the grf module.
plot.Font_Strings = 0
plot.text( "Hello", [0,75], [1,1], "CC" )

#  Change the definition of font 0 within the grf module, and plot
#  another string, still using font 0.
grf.fonts[ 0 ] = {"family":'monospace',"style":'italic'}
plot.text( "Hello", [0,65], [-1,1], "CC" )

#  Display everything.
plt.show()





#!/home/dsb/bin/python3

from __future__ import print_function

import sys
import starlink.Ast as Ast
import starlink.Grf as Grf
import matplotlib.pyplot as plt

#  Check the header name was supplied on the command line
if len( sys.argv ) < 2 :
   print( "Usage: plothead.py <basename>" )
   sys.exit(2)

#  Attempt to open the header file
fin1 = open( sys.argv[1]+".head" )

#  Attempt to open an associated file holding attributes that control how
#  the FITS headers are interpreted. If succesful, read the whole file into
#  a single string.
try:
   fin2 = open( sys.argv[1]+".fattr" )
   fits_atts = fin2.read()
   fin2.close()
except (IOError):
   fits_atts = ""

#  Attempt to open an associated file holding attributes that control the
#  appearance of the plot. If successful, read the whole file into a
#  single string.
try:
   fin2 = open( sys.argv[1]+".attr" )
   plot_atts = fin2.read()
   fin2.close()
except (IOError):
   plot_atts = ""

#  Attempt to open an associated file holding the pixel bounds of the
#  area of the FITS array to be plotted.
try:
   fin2 = open( sys.argv[1]+".box" )
   box = [float(v) for v in fin2.read().strip().split()]
   fin2.close()
except (IOError):
   box = None

#  Read the header lines into a list, and store this list in a new
#  Ast.FitsChan, using the requested attributes to modify the
#  interpretation of the header.
fc = Ast.FitsChan( fin1.readlines(), None, fits_atts )

#  Close the header file.
fin1.close()

#  Create a FrameSet from the FITS headers in the FitsChan.
fs = fc.read()
if fs == None:
   print( "Could not read WCS from headers in "+sys.argv[1]+".head" )
   sys.exit(2)

#  Exit if the header does not have exactly 2 pixel axes and 2 WCS axes.
if fs.Nin != 2 or fs.Nout != 2:
   print( "The headers in "+sys.argv[1]+".head do not describe a 2-D image" )
   sys.exit(2)

#  Create a matplotlib plotting region. Ensure that the matplotlib axis
#  annotations are not drawn.
dx=12.0
dy=9.0
fig = plt.figure( figsize=(dx,dy) )
ax = fig.add_axes( ( 0, 0, 1, 1 ) )
ax.xaxis.set_visible( False )
ax.yaxis.set_visible( False )

#  If the bounds of the pixel grid to be plotted were specified, use them
if box != None:
   bbox = box

#  Otherwise, we map the entire FITS pixel grid onto this box. So get the
#  bounds of the pixel grid from the FitsChan. If the NAXIS1/2 keywords
#  are not in the FitsChan, use defaults of 500.
else:
   if "NAXIS1" in fc:
      naxis1 = fc["NAXIS1"]
   else:
      naxis1 = 500

   if "NAXIS2" in fc:
      naxis2 = fc["NAXIS2"]
   else:
      naxis2 = 500
   bbox = ( 0.5, 0.5, naxis1 + 0.5, naxis2 + 0.5 )

#  Set the bounds (in matplotlib data coordinates) of the largest rectangle
#  that can be drawn on the matplotlib plotting area that has the same
#  aspect ratio as the FITS array.
fits_aspect_ratio = ( bbox[3] - bbox[1] )/( bbox[2] - bbox[0] )
if fits_aspect_ratio < 0.05 or fits_aspect_ratio > 20:
   fits_aspect_ratio = 1.0
grf_aspect_ratio = dy/dx
if grf_aspect_ratio > fits_aspect_ratio :
   hx = 0.5
   hy = 0.5*fits_aspect_ratio/grf_aspect_ratio
else:
   hx = 0.5*grf_aspect_ratio/fits_aspect_ratio
   hy = 0.5

#  Shrink the box to leave room for axis annotation.
hx *= 0.7
hy *= 0.7

gbox = ( 0.5 - hx, 0.5 - hy, 0.5 + hx, 0.5 + hy )

#  Create a drawing object that knows how to draw primitives (lines,
#  marks and strings) into the matplotlib plotting region.
grf = Grf.grf_matplotlib( ax )

#  Create the AST Plot, using the above object to draw the primitives. The
#  Plot is based on the FrameSet that describes the WCS read from the FITS
#  headers, so the plot knows how to convert from WCS coords to pixel
#  coords, and then to matplotlib data coords. Specify the plotting
#  attributes read from the "attr" file.
plot = Ast.Plot( fs, gbox, bbox, grf, plot_atts )

#  And finally, draw the annotated WCS axes and any coordinate grid
#  requested in the plotting attributes.
plot.grid()

#  Make the matplotlib plotting area visible
plt.show()




from __future__ import print_function

import pyfits
import starlink.Atl as Atl
import starlink.Ast as Ast
import matplotlib.pyplot

#  Use pyfits to open a test files file
ffile = pyfits.open( 'starlink/ast/test/cobe.fit' )

#  Use matplotlib to plot an annotated grid of the WCS coords
Atl.plotfitswcs( matplotlib.pyplot.figure(figsize=(8,8)).add_subplot(111),
                 [ 0.1, 0.1, 0.9, 0.9 ], ffile )
matplotlib.pyplot.show()

#  Create a FitsChan telling it to use the pyfits primary hdu as the
#  external data source and sink. Note, we take the default value of
#  "True" for the "clear" property when creating the PyFITSADapater,
#  which means the PyFITS header will be cleared immediately before
#  the FitsChan.writefits() method writes to it.
adapter = Atl.PyFITSAdapter(ffile)
fc = Ast.FitsChan( adapter, adapter )

#  Read the FrameSet from the FitsChan. This will read all headers from
#  the pyfits hdu into the FitsChan, create a FrameSet from the WCS
#  headers, and remove all WCS-related headers from the FitsChan (but not
#  the pyfits primary hdu as yet).
fs = fc.read()

#  Tell the FitsChan to write out the remaining headers to its external data
#  sink.
fc.writefits()

#  Display the headers now in the pyfits primary hdu.
print()
print("The non-WCS cards in cobe.fit: ")
for v in ffile[0].header.ascard:
   print(v)





from __future__ import print_function

from distutils import ccompiler
from distutils.core import setup, Extension
import sys, os, subprocess, numpy, tarfile
from tools import make_exceptions, make_attributes

include_dirs = []

include_dirs.append(numpy.get_include())
include_dirs.append(os.path.join('.','starlink','include'))
include_dirs.append(os.path.join('.','ast'))

#  Create the support files needed for the build. These find the AST
#  source code using the environment variable AST_SOURCE, so set AST_SOURCE
#  to point to the AST source code directory distributed with PyAST.
os.environ[ "AST_SOURCE" ] = os.path.join( os.getcwd(), 'ast' )
make_exceptions.make_exceptions( os.path.join('starlink','ast') )
make_attributes.make_attributes( os.path.join('starlink','ast') )

#  Extract the AST documentation
if not os.path.exists( 'sun211.htx'):
   tar = tarfile.open('ast/sun211.htx_tar')
   tar.extractall()
   tar.close()

#  List the C source files for implemeneted AST classes:
ast_c = ( 'axis.c', 'box.c', 'channel.c', 'circle.c', 'cmpframe.c',
          'cmpmap.c', 'cmpregion.c', 'dsbspecframe.c', 'dssmap.c',
	  'ellipse.c', 'error.c', 'fitschan.c', 'fluxframe.c', 'frame.c',
	  'frameset.c', 'globals.c', 'grf3d.c', 'grf_2.0.c', 'grf_3.2.c',
	  'grf_5.6.c', 'grismmap.c', 'interval.c', 'keymap.c',
	  'levmar.c', 'loader.c', 'lutmap.c', 'mapping.c', 'mathmap.c',
	  'matrixmap.c', 'memory.c', 'normmap.c', 'nullregion.c',
	  'object.c', 'pal.c', 'pcdmap.c', 'permmap.c', 'plot.c',
	  'pointlist.c', 'pointset.c', 'polygon.c', 'polymap.c',
	  'prism.c', 'proj.c', 'ratemap.c', 'region.c', 'shiftmap.c',
	  'skyaxis.c', 'skyframe.c', 'specfluxframe.c', 'specframe.c',
	  'sphmap.c', 'stcschan.c', 'timeframe.c', 'timemap.c', 'tpn.c',
	  'tranmap.c', 'unit.c', 'unitmap.c', 'wcsmap.c', 'wcstrig.c',
	  'winmap.c', 'xml.c', 'zoommap.c')

#  List the C source files for unimplemeneted AST classes:
ast_c_extra = ( 'fitstable.c', 'intramap.c', 'plot3d.c', 'selectormap.c',
                'slamap.c', 'specmap.c', 'stccatalogentrylocation.c',
		'stc.c', 'stcobsdatalocation.c', 'stcresourceprofile.c',
		'stcsearchlocation.c', 'switchmap.c', 'table.c',
		'xmlchan.c')

#  Initialise the list of sources files needed to build the starlink.Ast
#  module.
sources = [os.path.join('starlink', 'ast', 'Ast.c')]

#  Append all the .c and .h files needed to build the AST library locally.
for cfile in ast_c:
   sources.append( os.path.join( 'ast', cfile ) )
for cfile in ast_c_extra:
   sources.append( os.path.join( 'ast', cfile ) )

# Test the compiler
define_macros = []
compiler=ccompiler.new_compiler()
if compiler.has_function('strtok_r'):
   define_macros.append(('HAVE_STRTOK_R','1'))

if compiler.has_function('strerror_r'):
   define_macros.append(('HAVE_STRERROR_R','1'))

if compiler.has_function('isfinite'):
   define_macros.append(('HAVE_DECL_ISFINITE','1'))

#  We need to tell AST what type a 64-bit int will have
#  Not really sure how to determine whether we have int64_t
import ctypes

define_macros.append(('SIZEOF_LONG', ctypes.sizeof(ctypes.c_long)))
define_macros.append(('SIZEOF_LONG_LONG', ctypes.sizeof(ctypes.c_longlong)))

# Assume we have isnan() available and assume we have a working sscanf
# configure would test for these but we no longer run configure
define_macros.append(('HAVE_DECL_ISNAN','1'))

#  Create the description of the starlink.Ast module.
Ast = Extension('starlink.Ast',
                include_dirs  = include_dirs,
                define_macros = define_macros,
                sources       = sources )

# OSX needs to hide all the normal AST symbols to prevent
# name clashes when loaded alongside libast itself (eg from pyndf)
symbol_list = "public_symbols.txt"
if sys.platform.startswith("darwin"):
   symfile = open( symbol_list, "w" )
   if sys.version_info[0] > 2:
      symname = "_PyInit_Ast"
   else:
      symname = "_initAst"
   print(symname,file=symfile)
   symfile.close()
   Ast.extra_link_args = [ "-exported_symbols_list", symbol_list]


setup (name = 'starlink-pyast',
       version = '2.4',
       description = 'A Python wrapper for the Starlink AST library',
       url = 'http://starlink.jach.hawaii.edu/starlink/AST',
       author = 'David Berry',
       author_email = 'd.berry@jach.hawaii.edu',
       packages =['starlink'],
       package_data = { 'starlink': [os.path.join('include','star','pyast.h')] },
       ext_modules=[Ast],
       py_modules=['starlink.Grf','starlink.Atl'],
       classifiers=[
          'Intended Audience :: Developers',
          'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
          'Programming Language :: Python',
          'Programming Language :: C',
          'Topic :: Scientific/Engineering :: Astronomy'
       ])

if os.path.exists(symbol_list):
   os.unlink(symbol_list)
